! module with the differential equations
module hamiltonian

  use parameters
  implicit none

contains


!***************************************************************************************

  ! right-hand side of the equations of motion of the satellite's rotational and orbital motion
  ! assuming that the spin axis is parallel to the normal of the orbital plane
  function DEq(Hp,x)

     implicit none

     type(hamparam),intent(inOut):: Hp
     double precision,intent(in):: x(6)
     double precision:: DEq(6)

     double precision:: r,f

     !-----------------------------------------------
     r=sqrt(x(3)**2+x(4)**2)    ! r
     f=datan2(x(4),x(3))        ! true anomaly
     !-----------------------------------------------
     DEq = 0.d0

     ! rotation
     Deq(1)=x(2)
     Deq(2)=Hp%alpha**2/2.d0*(Hp%GM/r**3)*dsin(2.d0*(f-x(1)))
     ! tidal despinning
     IF(x(2)-Hp%n.gt.0.d0)THEN
        Deq(2)=Deq(2)-Hp%dotw
     ELSEIF(x(2)-Hp%n.lt.0.d0)THEN
        Deq(2)=Deq(2)+Hp%dotw
     ENDIF
     ! orbit (only plane components)
     Deq(3)=x(5)
     Deq(4)=x(6)
     Deq(5)=-Hp%GM/r**3*x(3)
     Deq(6)=-Hp%GM/r**3*x(4)
     !-----------------------------------------------

  end function DEq

!***************************************************************************************

end module hamiltonian
