! Simple program for the propagation of satellite's rotation
! course of Dinamica del Sistema Solare
program spinoDSS

  use hamiltonian
  use integr

  implicit none

  character(256):: hamParamFile ! namelist containing the parameters of the Hamiltonian function
  character(256):: intParamFile ! namelist containing the parameters of the numerical integration
  character(256):: outFile      ! output file

  type(hamparam):: Hp  ! parameters of the Hamiltonian
  type(intparam):: Ip  ! parameters of the numerical integration

  double precision:: X(6) ! state vector
  double precision:: t ! time
  integer:: Nstep, Nreg

!---------------------------------------------------------------------------------------------
! Reading and computing the parameters

  if(COMMAND_ARGUMENT_COUNT() >= 3) then
     call GET_COMMAND_ARGUMENT(1,hamParamFile)
     call GET_COMMAND_ARGUMENT(2,intParamFile)
     call GET_COMMAND_ARGUMENT(3,outFile)
  else
     write(*,*) "Use: <program name> <parameterFile.nml> <initialConditions.nml> <outFile.dat>"
     stop
  endIf

  ! computing the parameters of the Hamiltonian
  call readHamParam(hamParamFile,trim(outFile)//"_ham.nml",Hp)

  ! computing the parameters of the numerical integration
  call initInteg(intParamFile,trim(outFile)//"_int.nml", Hp,Ip, X)

!---------------------------------------------------------------------------------------------
! Initializations

  t = Ip%t0
  Nstep = 0 ! number of timesteps realised

  ! Opening and writing the header of the output file
  open(newunit=Nreg,file=trim(outFile),status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg,'(A)') "# theta is the rotation angle"
  write(Nreg,'(A)') "# omega/n is the spin velocity / mean motion"
  write(Nreg,'(A)') "# x is the first cartesian coordinate of the orbital position"
  write(Nreg,'(A)') "# y is the second cartesian coordinate of the orbital position"
  write(Nreg,'(A)') "# "

  write(Nreg,'(A,4(I0,A))') "# t ; theta; omega/n; x; y"

!---------------------------------------------------------------------------------------------
! Numerical integration
  call printFunc(X,t) ! saving the initial state
  call rkg(X, t, force, Ip%Dt, Ip%tSpan, printFunc) ! numerical integration

!---------------------------------------------------------------------------------------------

  close(Nreg) ! closing the output file

!---------------------------------------------------------------------------------------------
contains
!---------------------------------------------------------------------------------------------

  ! Force function for the numerical integration
  ! 
  function force(XX,tt)
     double precision,intent(in):: XX(:), tt
     double precision:: force(size(XX))

     force = DEq(Hp,XX)

  end function force

!---------------------------------------------------------------------------------------------

  ! Subroutine used after each step for saving the results.
  ! 
  subroutine printFunc(XX,tt)
     double precision,intent(in):: XX(:), tt

     if( mod(Nstep,Ip%Nsave) == 0 ) then
     
        write(Nreg,'(ES22.15,4(1X,E24.16))') tt, XX(1), XX(2)/Hp%n, XX(3), XX(4)

     endIf
     Nstep = Nstep + 1

  end subroutine printFunc

!---------------------------------------------------------------------------------------------

end program spinoDSS
